package dom

// AUTOGENERATED. DO NOT EDIT.

import (
	cdp "github.com/knq/chromedp/cdp"
)

// EventDocumentUpdated fired when Document has been totally updated. Node
// ids are no longer valid.
type EventDocumentUpdated struct{}

// EventInspectNodeRequested fired when the node should be inspected. This
// happens after call to setInspectMode.
type EventInspectNodeRequested struct {
	BackendNodeID cdp.BackendNodeID `json:"backendNodeId,omitempty"` // Id of the node to inspect.
}

// EventSetChildNodes fired when backend wants to provide client with the
// missing DOM structure. This happens upon most of the calls requesting node
// ids.
type EventSetChildNodes struct {
	ParentID cdp.NodeID  `json:"parentId,omitempty"` // Parent node id to populate with children.
	Nodes    []*cdp.Node `json:"nodes,omitempty"`    // Child nodes array.
}

// EventAttributeModified fired when Element's attribute is modified.
type EventAttributeModified struct {
	NodeID cdp.NodeID `json:"nodeId,omitempty"` // Id of the node that has changed.
	Name   string     `json:"name,omitempty"`   // Attribute name.
	Value  string     `json:"value,omitempty"`  // Attribute value.
}

// EventAttributeRemoved fired when Element's attribute is removed.
type EventAttributeRemoved struct {
	NodeID cdp.NodeID `json:"nodeId,omitempty"` // Id of the node that has changed.
	Name   string     `json:"name,omitempty"`   // A ttribute name.
}

// EventInlineStyleInvalidated fired when Element's inline style is modified
// via a CSS property modification.
type EventInlineStyleInvalidated struct {
	NodeIds []cdp.NodeID `json:"nodeIds,omitempty"` // Ids of the nodes for which the inline styles have been invalidated.
}

// EventCharacterDataModified mirrors DOMCharacterDataModified event.
type EventCharacterDataModified struct {
	NodeID        cdp.NodeID `json:"nodeId,omitempty"`        // Id of the node that has changed.
	CharacterData string     `json:"characterData,omitempty"` // New text value.
}

// EventChildNodeCountUpdated fired when Container's child node count has
// changed.
type EventChildNodeCountUpdated struct {
	NodeID         cdp.NodeID `json:"nodeId,omitempty"`         // Id of the node that has changed.
	ChildNodeCount int64      `json:"childNodeCount,omitempty"` // New node count.
}

// EventChildNodeInserted mirrors DOMNodeInserted event.
type EventChildNodeInserted struct {
	ParentNodeID   cdp.NodeID `json:"parentNodeId,omitempty"`   // Id of the node that has changed.
	PreviousNodeID cdp.NodeID `json:"previousNodeId,omitempty"` // If of the previous siblint.
	Node           *cdp.Node  `json:"node,omitempty"`           // Inserted node data.
}

// EventChildNodeRemoved mirrors DOMNodeRemoved event.
type EventChildNodeRemoved struct {
	ParentNodeID cdp.NodeID `json:"parentNodeId,omitempty"` // Parent id.
	NodeID       cdp.NodeID `json:"nodeId,omitempty"`       // Id of the node that has been removed.
}

// EventShadowRootPushed called when shadow root is pushed into the element.
type EventShadowRootPushed struct {
	HostID cdp.NodeID `json:"hostId,omitempty"` // Host element id.
	Root   *cdp.Node  `json:"root,omitempty"`   // Shadow root.
}

// EventShadowRootPopped called when shadow root is popped from the element.
type EventShadowRootPopped struct {
	HostID cdp.NodeID `json:"hostId,omitempty"` // Host element id.
	RootID cdp.NodeID `json:"rootId,omitempty"` // Shadow root id.
}

// EventPseudoElementAdded called when a pseudo element is added to an
// element.
type EventPseudoElementAdded struct {
	ParentID      cdp.NodeID `json:"parentId,omitempty"`      // Pseudo element's parent element id.
	PseudoElement *cdp.Node  `json:"pseudoElement,omitempty"` // The added pseudo element.
}

// EventPseudoElementRemoved called when a pseudo element is removed from an
// element.
type EventPseudoElementRemoved struct {
	ParentID        cdp.NodeID `json:"parentId,omitempty"`        // Pseudo element's parent element id.
	PseudoElementID cdp.NodeID `json:"pseudoElementId,omitempty"` // The removed pseudo element id.
}

// EventDistributedNodesUpdated called when distrubution is changed.
type EventDistributedNodesUpdated struct {
	InsertionPointID cdp.NodeID         `json:"insertionPointId,omitempty"` // Insertion point where distrubuted nodes were updated.
	DistributedNodes []*cdp.BackendNode `json:"distributedNodes,omitempty"` // Distributed nodes for given insertion point.
}

// EventNodeHighlightRequested [no description].
type EventNodeHighlightRequested struct {
	NodeID cdp.NodeID `json:"nodeId,omitempty"`
}

// EventTypes all event types in the domain.
var EventTypes = []cdp.MethodType{
	cdp.EventDOMDocumentUpdated,
	cdp.EventDOMInspectNodeRequested,
	cdp.EventDOMSetChildNodes,
	cdp.EventDOMAttributeModified,
	cdp.EventDOMAttributeRemoved,
	cdp.EventDOMInlineStyleInvalidated,
	cdp.EventDOMCharacterDataModified,
	cdp.EventDOMChildNodeCountUpdated,
	cdp.EventDOMChildNodeInserted,
	cdp.EventDOMChildNodeRemoved,
	cdp.EventDOMShadowRootPushed,
	cdp.EventDOMShadowRootPopped,
	cdp.EventDOMPseudoElementAdded,
	cdp.EventDOMPseudoElementRemoved,
	cdp.EventDOMDistributedNodesUpdated,
	cdp.EventDOMNodeHighlightRequested,
}
